@@ -24,27 +24,14 @@
  *****************************************************************************/
 #include "rtl_ps.h"
 #include "rtl_core.h"
-#ifdef RTL8192SE
-#include "rtl_core.h"
-#include "r8192S_phy.h"
-#include "r8192S_phyreg.h"
-#include "r8192S_rtl6052.h"
-#include "r8192S_Efuse.h"
-#else
-#include "rtl_core.h"
-#include "r819xE_phy.h"
-#include "r819xE_phyreg.h"
-#include "r8190_rtl8256.h" /* RTL8225 Radio frontend */
-#include "r819xE_cmdpkt.h"
-#endif
 
-#if defined(RTL8192E) || defined(RTL8192SE)
+#if defined(RTL8192E) || defined(RTL8192SE) || defined RTL8192CE
 void rtl8192_hw_sleep_down(struct net_device *dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
+	struct r8192_priv *priv = rtllib_priv(dev);
 	unsigned long flags = 0;
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
-	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+#ifdef CONFIG_ASPM_OR_D3
+	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 #endif	
 	spin_lock_irqsave(&priv->rf_ps_lock,flags);
 	if (priv->RFChangeInProgress) {
@@ -61,8 +48,8 @@ void rtl8192_hw_sleep_down(struct net_device *dev)
 		return;
 	}
 #endif	
-	MgntActSet_RF_State(dev, eRfSleep, RF_CHANGE_BY_PS);
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
+	MgntActSet_RF_State(dev, eRfSleep, RF_CHANGE_BY_PS,false);
+#ifdef CONFIG_ASPM_OR_D3
 	if(pPSC->RegRfPsLevel & RT_RF_LPS_LEVEL_ASPM)
 	{
 		RT_ENABLE_ASPM(dev);
@@ -74,7 +61,7 @@ void rtl8192_hw_sleep_down(struct net_device *dev)
 void rtl8192_hw_sleep_wq(void *data)
 {
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20))
-	struct ieee80211_device *ieee = container_of_dwork_rsl(data,struct ieee80211_device,hw_sleep_wq);
+	struct rtllib_device *ieee = container_of_dwork_rsl(data,struct rtllib_device,hw_sleep_wq);
 	struct net_device *dev = ieee->dev;
 #else
 	struct net_device *dev = (struct net_device *)data;
@@ -84,34 +71,34 @@ void rtl8192_hw_sleep_wq(void *data)
 
 void rtl8192_hw_wakeup(struct net_device* dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
+	struct r8192_priv *priv = rtllib_priv(dev);
 	unsigned long flags = 0;
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
-	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+#ifdef CONFIG_ASPM_OR_D3
+	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 #endif	
 	spin_lock_irqsave(&priv->rf_ps_lock,flags);
 	if (priv->RFChangeInProgress) {
 		spin_unlock_irqrestore(&priv->rf_ps_lock,flags);
 		RT_TRACE(COMP_RF, "rtl8192_hw_wakeup(): RF Change in progress! \n");
 		printk("rtl8192_hw_wakeup(): RF Change in progress! schedule wake up task again\n");
-		queue_delayed_work_rsl(priv->ieee80211->wq,&priv->ieee80211->hw_wakeup_wq,MSECS(10));
+		queue_delayed_work_rsl(priv->rtllib->wq,&priv->rtllib->hw_wakeup_wq,MSECS(10));
 		return;
 	}
 	spin_unlock_irqrestore(&priv->rf_ps_lock,flags);
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
+#ifdef CONFIG_ASPM_OR_D3
 	if (pPSC->RegRfPsLevel & RT_RF_LPS_LEVEL_ASPM) {
 		RT_DISABLE_ASPM(dev);
 		RT_CLEAR_PS_LEVEL(pPSC, RT_RF_LPS_LEVEL_ASPM);
 	}
 #endif
 	RT_TRACE(COMP_PS, "%s()============>come to wake up\n", __FUNCTION__);
-	MgntActSet_RF_State(dev, eRfOn, RF_CHANGE_BY_PS);
+	MgntActSet_RF_State(dev, eRfOn, RF_CHANGE_BY_PS,false);
 }
 
 void rtl8192_hw_wakeup_wq(void *data)
 {
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20))
-	struct ieee80211_device *ieee = container_of_dwork_rsl(data,struct ieee80211_device,hw_wakeup_wq);  
+	struct rtllib_device *ieee = container_of_dwork_rsl(data,struct rtllib_device,hw_wakeup_wq);  
 	struct net_device *dev = ieee->dev;
 #else
 	struct net_device *dev = (struct net_device *)data;
@@ -125,7 +112,7 @@ void rtl8192_hw_wakeup_wq(void *data)
 void rtl8192_hw_to_sleep(struct net_device *dev, u32 th, u32 tl)
 {
 #ifdef _RTL8192_EXT_PATCH_
-	struct r8192_priv *priv = ieee80211_priv(dev);
+	struct r8192_priv *priv = rtllib_priv(dev);
 	u32 rb = jiffies, sleep_cost = MSECS(8+16+7), delta = 0;
 	unsigned long flags;
 
@@ -166,12 +153,13 @@ void rtl8192_hw_to_sleep(struct net_device *dev, u32 th, u32 tl)
 		return;
 	}
 
-	queue_delayed_work_rsl(priv->ieee80211->wq,&priv->ieee80211->hw_wakeup_wq,delta); 
-	queue_delayed_work_rsl(priv->ieee80211->wq, (void *)&priv->ieee80211->hw_sleep_wq,0);
+	RT_TRACE(COMP_LPS, "==============>%s(): wake up time is %d,%d\n",__FUNCTION__,delta,jiffies_to_msecs(delta));
+	queue_delayed_work_rsl(priv->rtllib->wq,&priv->rtllib->hw_wakeup_wq,delta); 
+	queue_delayed_work_rsl(priv->rtllib->wq, (void *)&priv->rtllib->hw_sleep_wq,0);
 
 	spin_unlock_irqrestore(&priv->ps_lock,flags);
 #else
-	struct r8192_priv *priv = ieee80211_priv(dev);
+	struct r8192_priv *priv = rtllib_priv(dev);
 
 	u32 rb = jiffies;
 	unsigned long flags;
@@ -196,11 +184,11 @@ void rtl8192_hw_to_sleep(struct net_device *dev, u32 th, u32 tl)
 	}
 	{
 		u32 tmp = (tl>rb)?(tl-rb):(rb-tl);
-		queue_delayed_work_rsl(priv->ieee80211->wq,
-				&priv->ieee80211->hw_wakeup_wq,tmp); 
+		queue_delayed_work_rsl(priv->rtllib->wq,
+				&priv->rtllib->hw_wakeup_wq,tmp); 
 	}
-	queue_delayed_work_rsl(priv->ieee80211->wq, 
-			(void *)&priv->ieee80211->hw_sleep_wq,0);
+	queue_delayed_work_rsl(priv->rtllib->wq, 
+			(void *)&priv->rtllib->hw_sleep_wq,0);
 	spin_unlock_irqrestore(&priv->ps_lock,flags);
 #endif
 }
@@ -209,15 +197,15 @@ void rtl8192_hw_to_sleep(struct net_device *dev, u32 th, u32 tl)
 #ifdef ENABLE_IPS
 void InactivePsWorkItemCallback(struct net_device *dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+	struct r8192_priv *priv = rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 
 	RT_TRACE(COMP_PS, "InactivePsWorkItemCallback() ---------> \n");			
 	pPSC->bSwRfProcessing = true;
 
 	RT_TRACE(COMP_PS, "InactivePsWorkItemCallback(): Set RF to %s.\n", \
 			pPSC->eInactivePowerState == eRfOff?"OFF":"ON");
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
+#ifdef CONFIG_ASPM_OR_D3
 	if(pPSC->eInactivePowerState == eRfOn)
 	{
 
@@ -235,9 +223,9 @@ void InactivePsWorkItemCallback(struct net_device *dev)
 #endif		
 	}
 #endif
-	MgntActSet_RF_State(dev, pPSC->eInactivePowerState, RF_CHANGE_BY_IPS);
+	MgntActSet_RF_State(dev, pPSC->eInactivePowerState, RF_CHANGE_BY_IPS,false);
 
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
+#ifdef CONFIG_ASPM_OR_D3
 	if(pPSC->eInactivePowerState == eRfOff)
 	{
 		if(pPSC->RegRfPsLevel & RT_RF_OFF_LEVL_ASPM)
@@ -278,16 +266,16 @@ void InactivePsWorkItemCallback(struct net_device *dev)
 void
 IPSEnter(struct net_device *dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL		pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+	struct r8192_priv *priv = rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL		pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 	RT_RF_POWER_STATE 			rtState;
 
 	if (pPSC->bInactivePs)
 	{
-		rtState = priv->ieee80211->eRFPowerState;
+		rtState = priv->rtllib->eRFPowerState;
 		if (rtState == eRfOn && !pPSC->bSwRfProcessing &&\
-			(priv->ieee80211->state != IEEE80211_LINKED)&&\
-			(priv->ieee80211->iw_mode != IW_MODE_MASTER))
+			(priv->rtllib->state != RTLLIB_LINKED)&&\
+			(priv->rtllib->iw_mode != IW_MODE_MASTER))
 		{
 			RT_TRACE(COMP_PS,"IPSEnter(): Turn off RF.\n");
 			pPSC->eInactivePowerState = eRfOff;
@@ -301,14 +289,14 @@ IPSEnter(struct net_device *dev)
 void
 IPSLeave(struct net_device *dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+	struct r8192_priv *priv = rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 	RT_RF_POWER_STATE 	rtState;
 
 	if (pPSC->bInactivePs)
 	{	
-		rtState = priv->ieee80211->eRFPowerState;	
-		if (rtState != eRfOn  && !pPSC->bSwRfProcessing && priv->ieee80211->RfOffReason <= RF_CHANGE_BY_IPS)
+		rtState = priv->rtllib->eRFPowerState;	
+		if (rtState != eRfOn  && !pPSC->bSwRfProcessing && priv->rtllib->RfOffReason <= RF_CHANGE_BY_IPS)
 		{
 			RT_TRACE(COMP_PS, "IPSLeave(): Turn on RF.\n");
 			pPSC->eInactivePowerState = eRfOn;
@@ -320,55 +308,55 @@ IPSLeave(struct net_device *dev)
 void IPSLeave_wq(void *data)
 {
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20))
-	struct ieee80211_device *ieee = container_of_work_rsl(data,struct ieee80211_device,ips_leave_wq);
+	struct rtllib_device *ieee = container_of_work_rsl(data,struct rtllib_device,ips_leave_wq);
 	struct net_device *dev = ieee->dev;
 #else
 	struct net_device *dev = (struct net_device *)data;
 #endif
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
-	down(&priv->ieee80211->ips_sem);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
+	down(&priv->rtllib->ips_sem);
 	IPSLeave(dev);	
-	up(&priv->ieee80211->ips_sem);	
+	up(&priv->rtllib->ips_sem);	
 }
-void ieee80211_ips_leave_wq(struct net_device *dev)
+void rtllib_ips_leave_wq(struct net_device *dev)
 {
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
 	RT_RF_POWER_STATE	rtState;
-	rtState = priv->ieee80211->eRFPowerState;
+	rtState = priv->rtllib->eRFPowerState;
 
-	if(priv->ieee80211->PowerSaveControl.bInactivePs){ 
+	if(priv->rtllib->PowerSaveControl.bInactivePs){ 
 		if(rtState == eRfOff){
-			if(priv->ieee80211->RfOffReason > RF_CHANGE_BY_IPS)
+			if(priv->rtllib->RfOffReason > RF_CHANGE_BY_IPS)
 			{
 				RT_TRACE(COMP_ERR, "%s(): RF is OFF.\n",__FUNCTION__);
 				return;
 			}
 			else{
 				printk("=========>%s(): IPSLeave\n",__FUNCTION__);
-				queue_work_rsl(priv->ieee80211->wq,&priv->ieee80211->ips_leave_wq);				
+				queue_work_rsl(priv->rtllib->wq,&priv->rtllib->ips_leave_wq);				
 			}
 		}
 	}
 }
-void ieee80211_ips_leave(struct net_device *dev)
+void rtllib_ips_leave(struct net_device *dev)
 {
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
-	down(&priv->ieee80211->ips_sem);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
+	down(&priv->rtllib->ips_sem);
 	IPSLeave(dev);	
-	up(&priv->ieee80211->ips_sem);	
+	up(&priv->rtllib->ips_sem);	
 }
 #endif
 
 #ifdef ENABLE_LPS
 bool MgntActSet_802_11_PowerSaveMode(struct net_device *dev,	u8 rtPsMode)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
+	struct r8192_priv *priv = rtllib_priv(dev);
 
 #ifdef _RTL8192_EXT_PATCH_
-	if((priv->ieee80211->iw_mode == IW_MODE_ADHOC) || (priv->ieee80211->iw_mode == IW_MODE_MASTER)
-		|| (priv->ieee80211->iw_mode == IW_MODE_MESH))	
+	if((priv->rtllib->iw_mode == IW_MODE_ADHOC) || (priv->rtllib->iw_mode == IW_MODE_MASTER)
+		|| (priv->rtllib->iw_mode == IW_MODE_MESH))	
 #else
-	if(priv->ieee80211->iw_mode == IW_MODE_ADHOC)	
+	if(priv->rtllib->iw_mode == IW_MODE_ADHOC)	
 #endif
 	{
 		return false;
@@ -377,35 +365,35 @@ bool MgntActSet_802_11_PowerSaveMode(struct net_device *dev,	u8 rtPsMode)
 	
 	RT_TRACE(COMP_LPS,"%s(): set ieee->ps = %x\n",__FUNCTION__,rtPsMode);
 	if(!priv->ps_force) {
-		priv->ieee80211->ps = rtPsMode;
+		priv->rtllib->ps = rtPsMode;
 	}
 #if 0
-	priv->ieee80211->dot11PowerSaveMode = rtPsMode;
+	priv->rtllib->dot11PowerSaveMode = rtPsMode;
 
-	if(priv->ieee80211->dot11PowerSaveMode == eMaxPs)
+	if(priv->rtllib->dot11PowerSaveMode == eMaxPs)
 	{
 	}
 	else
 	{
 	}
 #endif
-	if(priv->ieee80211->sta_sleep != 0 && rtPsMode == IEEE80211_PS_DISABLED)
+	if(priv->rtllib->sta_sleep != LPS_IS_WAKE && rtPsMode == RTLLIB_PS_DISABLED)
 	{
                 unsigned long flags;
 
 		rtl8192_hw_wakeup(dev);
-		priv->ieee80211->sta_sleep = 0;
+		priv->rtllib->sta_sleep = LPS_IS_WAKE;
 
-                spin_lock_irqsave(&(priv->ieee80211->mgmt_tx_lock), flags);
+                spin_lock_irqsave(&(priv->rtllib->mgmt_tx_lock), flags);
 		printk("LPS leave: notify AP we are awaked ++++++++++ SendNullFunctionData\n");
-		ieee80211_sta_ps_send_null_frame(priv->ieee80211, 0);
-                spin_unlock_irqrestore(&(priv->ieee80211->mgmt_tx_lock), flags);
+		rtllib_sta_ps_send_null_frame(priv->rtllib, 0);
+                spin_unlock_irqrestore(&(priv->rtllib->mgmt_tx_lock), flags);
 	}
 
 #if 0
 	if((pPSC->bFwCtrlLPS) && (pPSC->bLeisurePs))
 	{	
-		if(priv->ieee80211->dot11PowerSaveMode == eActive)
+		if(priv->rtllib->dot11PowerSaveMode == eActive)
 		{
 			RpwmVal = 0x0C; 
 			FwPwrMode = FW_PS_ACTIVE_MODE;
@@ -434,20 +422,20 @@ bool MgntActSet_802_11_PowerSaveMode(struct net_device *dev,	u8 rtPsMode)
 
 void LeisurePSEnter(struct net_device *dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+	struct r8192_priv *priv = rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 
 	RT_TRACE(COMP_PS, "LeisurePSEnter()...\n");
 	RT_TRACE(COMP_PS, "pPSC->bLeisurePs = %d, ieee->ps = %d,pPSC->LpsIdleCount is %d,RT_CHECK_FOR_HANG_PERIOD is %d\n", 
-		pPSC->bLeisurePs, priv->ieee80211->ps,pPSC->LpsIdleCount,RT_CHECK_FOR_HANG_PERIOD);
+		pPSC->bLeisurePs, priv->rtllib->ps,pPSC->LpsIdleCount,RT_CHECK_FOR_HANG_PERIOD);
 
 #ifdef _RTL8192_EXT_PATCH_
-	if(!((priv->ieee80211->iw_mode == IW_MODE_INFRA) && (priv->ieee80211->state == IEEE80211_LINKED))
-		|| (priv->ieee80211->iw_mode == IW_MODE_ADHOC) || (priv->ieee80211->iw_mode == IW_MODE_MASTER)
-		|| (priv->ieee80211->iw_mode == IW_MODE_MESH))
+	if(!((priv->rtllib->iw_mode == IW_MODE_INFRA) && (priv->rtllib->state == RTLLIB_LINKED))
+		|| (priv->rtllib->iw_mode == IW_MODE_ADHOC) || (priv->rtllib->iw_mode == IW_MODE_MASTER)
+		|| (priv->rtllib->iw_mode == IW_MODE_MESH))
 #else
-	if(!((priv->ieee80211->iw_mode == IW_MODE_INFRA) && (priv->ieee80211->state == IEEE80211_LINKED))
-		|| (priv->ieee80211->iw_mode == IW_MODE_ADHOC) || (priv->ieee80211->iw_mode == IW_MODE_MASTER))
+	if(!((priv->rtllib->iw_mode == IW_MODE_INFRA) && (priv->rtllib->state == RTLLIB_LINKED))
+		|| (priv->rtllib->iw_mode == IW_MODE_ADHOC) || (priv->rtllib->iw_mode == IW_MODE_MASTER))
 #endif
 		return;
 
@@ -456,19 +444,19 @@ void LeisurePSEnter(struct net_device *dev)
 		if(pPSC->LpsIdleCount >= RT_CHECK_FOR_HANG_PERIOD) 
 		{
 	
-			if(priv->ieee80211->ps == IEEE80211_PS_DISABLED)
+			if(priv->rtllib->ps == RTLLIB_PS_DISABLED)
 			{
 
 				RT_TRACE(COMP_LPS, "LeisurePSEnter(): Enter 802.11 power save mode...\n");
 
 				if(!pPSC->bFwCtrlLPS)
 				{
-					if (priv->ieee80211->SetFwCmdHandler)
+					if (priv->rtllib->SetFwCmdHandler)
 					{
-						priv->ieee80211->SetFwCmdHandler(dev, FW_CMD_LPS_ENTER);
+						priv->rtllib->SetFwCmdHandler(dev, FW_CMD_LPS_ENTER);
 					} 
 				}	
-				MgntActSet_802_11_PowerSaveMode(dev, IEEE80211_PS_MBCAST|IEEE80211_PS_UNICAST);
+				MgntActSet_802_11_PowerSaveMode(dev, RTLLIB_PS_MBCAST|RTLLIB_PS_UNICAST);
 
 				/*if(pPSC->RegRfPsLevel & RT_RF_LPS_LEVEL_ASPM)
 				{
@@ -486,19 +474,19 @@ void LeisurePSEnter(struct net_device *dev)
 
 void LeisurePSLeave(struct net_device *dev)
 {
-	struct r8192_priv *priv = ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+	struct r8192_priv *priv = rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 
 
 	RT_TRACE(COMP_PS, "LeisurePSLeave()...\n");
 	RT_TRACE(COMP_PS, "pPSC->bLeisurePs = %d, ieee->ps = %d\n", 
-		pPSC->bLeisurePs, priv->ieee80211->ps);
+		pPSC->bLeisurePs, priv->rtllib->ps);
 
 	if (pPSC->bLeisurePs)
 	{	
-		if(priv->ieee80211->ps != IEEE80211_PS_DISABLED)
+		if(priv->rtllib->ps != RTLLIB_PS_DISABLED)
 		{
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
+#ifdef CONFIG_ASPM_OR_D3
 			if(pPSC->RegRfPsLevel & RT_RF_LPS_LEVEL_ASPM && RT_IN_PS_LEVEL(pPSC, RT_RF_LPS_LEVEL_ASPM))
 			{
 				RT_DISABLE_ASPM(dev);
@@ -506,13 +494,13 @@ void LeisurePSLeave(struct net_device *dev)
 			} 
 #endif
 			RT_TRACE(COMP_LPS, "LeisurePSLeave(): Busy Traffic , Leave 802.11 power save..\n");
-			MgntActSet_802_11_PowerSaveMode(dev, IEEE80211_PS_DISABLED);
+			MgntActSet_802_11_PowerSaveMode(dev, RTLLIB_PS_DISABLED);
 
 			if(!pPSC->bFwCtrlLPS) 
 			{
-				if (priv->ieee80211->SetFwCmdHandler)
+				if (priv->rtllib->SetFwCmdHandler)
 				{
-					priv->ieee80211->SetFwCmdHandler(dev, FW_CMD_LPS_LEAVE);
+					priv->rtllib->SetFwCmdHandler(dev, FW_CMD_LPS_LEAVE);
 				} 
                     }
 		}
@@ -520,14 +508,95 @@ void LeisurePSLeave(struct net_device *dev)
 }
 #endif
 
-#ifdef RTL8192SE_CONFIG_ASPM_OR_D3
+#ifdef CONFIG_ASPM_OR_D3
+
+void
+PlatformDisableHostL0s(struct net_device *dev)
+{
+	struct r8192_priv 	*priv = (struct r8192_priv *)rtllib_priv(dev);
+	u32				PciCfgAddrPort=0;
+	u8				Num4Bytes;
+	u8				uPciBridgeASPMSetting = 0;
+
+	
+	if( (priv->NdisAdapter.BusNumber == 0xff && priv->NdisAdapter.DevNumber == 0xff && priv->NdisAdapter.FuncNumber == 0xff) ||
+		(priv->NdisAdapter.PciBridgeBusNum == 0xff && priv->NdisAdapter.PciBridgeDevNum == 0xff && priv->NdisAdapter.PciBridgeFuncNum == 0xff) )
+	{
+		printk("PlatformDisableHostL0s(): Fail to enable ASPM. Cannot find the Bus of PCI(Bridge).\n");
+		return;
+	}
+	
+	PciCfgAddrPort= (priv->NdisAdapter.PciBridgeBusNum << 16)|(priv->NdisAdapter.PciBridgeDevNum<< 11)|(priv->NdisAdapter.PciBridgeFuncNum <<  8)|(1 << 31);
+	Num4Bytes = (priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10)/4;
+
+
+	NdisRawWritePortUlong(PCI_CONF_ADDRESS , PciCfgAddrPort+(Num4Bytes << 2));
+
+	NdisRawReadPortUchar(PCI_CONF_DATA, &uPciBridgeASPMSetting);
+
+	if(uPciBridgeASPMSetting & BIT0)
+		uPciBridgeASPMSetting &=  ~(BIT0);
+
+	NdisRawWritePortUlong(PCI_CONF_ADDRESS , PciCfgAddrPort+(Num4Bytes << 2));
+	NdisRawWritePortUchar(PCI_CONF_DATA, uPciBridgeASPMSetting);
+
+	udelay(50);
+
+	printk("PlatformDisableHostL0s():PciBridge BusNumber[%x], DevNumbe[%x], FuncNumber[%x], Write reg[%x] = %x\n",
+		priv->NdisAdapter.PciBridgeBusNum, priv->NdisAdapter.PciBridgeDevNum, priv->NdisAdapter.PciBridgeFuncNum, 
+		(priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10), (priv->NdisAdapter.PciBridgeLinkCtrlReg | (priv->RegDevicePciASPMSetting&~BIT0)));
+}
+
+bool
+PlatformEnable92CEBackDoor(struct net_device *dev)
+{
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
+	bool			bResult = true;
+	u8			value;
+
+	if( (priv->NdisAdapter.BusNumber == 0xff && priv->NdisAdapter.DevNumber == 0xff && priv->NdisAdapter.FuncNumber == 0xff) ||
+		(priv->NdisAdapter.PciBridgeBusNum == 0xff && priv->NdisAdapter.PciBridgeDevNum == 0xff && priv->NdisAdapter.PciBridgeFuncNum == 0xff) )
+	{
+		RT_TRACE(COMP_INIT, "PlatformEnableASPM(): Fail to enable ASPM. Cannot find the Bus of PCI(Bridge).\n");
+		return false;
+	}
+	
+	pci_read_config_byte(priv->pdev, 0x70f, &value);
+	
+	if(priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_INTEL)
+	{
+	value |= BIT7;
+	}
+	else
+	{
+		value = 0x23;
+	}
+	
+	pci_write_config_byte(priv->pdev, 0x70f, value);
+
+
+	pci_read_config_byte(priv->pdev, 0x719, &value);
+	value |= (BIT3|BIT4);
+	pci_write_config_byte(priv->pdev, 0x719, value);
+	
+
+	return bResult;
+}
+
 bool PlatformSwitchDevicePciASPM(struct net_device *dev, u8 value)
 {
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
 	bool bResult = false;
 
-	pci_write_config_byte(priv->pdev, priv->ASPMRegOffset, value);
+#ifdef RTL8192CE
+	value |= 0x40;
+#endif
+
+	pci_write_config_byte(priv->pdev, 0x80, value);
+
+#ifdef RTL8192SE
 	udelay(100);
+#endif
 
 	return bResult;
 }
@@ -535,13 +604,20 @@ bool PlatformSwitchDevicePciASPM(struct net_device *dev, u8 value)
 bool PlatformSwitchClkReq(struct net_device *dev, u8 value)
 {
 	bool bResult = false;
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
 	u8	Buffer;
 
 	Buffer= value;	
 	
-	pci_write_config_byte(priv->pdev,priv->ClkReqOffset,value);
+#ifdef MERGE_TO_DO
+	if(Adapter->bDriverIsGoingToPnpSetPowerSleep && pDevice->RegSupportLowPowerState
+		&& value == 0x0)
+		return false;
+#endif
+
+	pci_write_config_byte(priv->pdev,0x81,value);
 	bResult = true;
+	
 #ifdef TODO
 	if(Buffer) {
 		priv->ClkReqState = true;
@@ -549,55 +625,181 @@ bool PlatformSwitchClkReq(struct net_device *dev, u8 value)
 		priv->ClkReqState = false;
 	}
 #endif
+
+#ifdef RTL8192SE
 	udelay(100);
+#endif
+
 	return bResult;
 }
 
 void
 PlatformDisableASPM(struct net_device *dev)
 {
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
-
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL	pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
+#if 1
+	u32	PciCfgAddrPort=0;
+	u8	Num4Bytes;
+#endif
 	u8	LinkCtrlReg;
 	u16	PciBridgeLinkCtrlReg, ASPMLevel=0;
 
-	LinkCtrlReg = priv->LinkCtrlReg;
+#ifdef RTL8192CE
+	if( (priv->NdisAdapter.BusNumber == 0xff && priv->NdisAdapter.DevNumber == 0xff && priv->NdisAdapter.FuncNumber == 0xff) ||
+		(priv->NdisAdapter.PciBridgeBusNum == 0xff && priv->NdisAdapter.PciBridgeDevNum == 0xff && priv->NdisAdapter.PciBridgeFuncNum == 0xff) )
+	{
+		RT_TRACE(COMP_INIT, "PlatformEnableASPM(): Fail to enable ASPM. Cannot find the Bus of PCI(Bridge).\n");
+		return;
+	}
+#endif
+
+#ifdef RTL8192SE
+	if(priv->NdisAdapter.PciBridgeVendor != PCI_BRIDGE_VENDOR_INTEL )
+	{
+		RT_TRACE(COMP_POWER, "%s(): Dont modify ASPM for non intel chipset. For Bridge Vendor %d.\n"
+			,__func__,priv->NdisAdapter.PciBridgeVendor);
+		return;
+	}
+#endif
+
+	if(priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_UNKNOWN)
+	{
+		RT_TRACE(COMP_POWER,  "%s(): Disable ASPM. Recognize the Bus of PCI(Bridge) as UNKNOWN.\n",__func__);
+	}
+
+#ifdef RTL8192CE
+	if(pPSC->RegRfPsLevel & RT_RF_OFF_LEVL_CLK_REQ)
+	{
+		RT_CLEAR_PS_LEVEL(pPSC, RT_RF_OFF_LEVL_CLK_REQ);
+		PlatformSwitchClkReq(dev, 0x0);
+	}	
+
+	{
+		u8	tmpU1b;
+	
+		pci_read_config_byte(priv->pdev, 0x80, &tmpU1b);		
+	}
+#endif
+
+	LinkCtrlReg = priv->NdisAdapter.LinkCtrlReg;
+	PciBridgeLinkCtrlReg = priv->NdisAdapter.PciBridgeLinkCtrlReg;
+
 	ASPMLevel |= BIT0|BIT1;
 	LinkCtrlReg &=~ASPMLevel;
-
-	PciBridgeLinkCtrlReg = priv->PciBridgeLinkCtrlReg;
 	PciBridgeLinkCtrlReg &=~(BIT0|BIT1);
 
-	if (priv->aspm_clkreq_enable) {
+#ifdef RTL8192CE
+	PlatformSwitchDevicePciASPM(dev, LinkCtrlReg);
+	udelay( 50);
+#endif
 
+	if( (priv->NdisAdapter.BusNumber == 0xff && priv->NdisAdapter.DevNumber == 0xff && priv->NdisAdapter.FuncNumber == 0xff) ||
+		(priv->NdisAdapter.PciBridgeBusNum == 0xff && priv->NdisAdapter.PciBridgeDevNum == 0xff && priv->NdisAdapter.PciBridgeFuncNum == 0xff) )
+	{
+	} else  {
+#if 1
+		PciCfgAddrPort= (priv->NdisAdapter.PciBridgeBusNum << 16)|(priv->NdisAdapter.PciBridgeDevNum<< 11)|(priv->NdisAdapter.PciBridgeFuncNum <<  8)|(1 << 31);
+		Num4Bytes = (priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10)/4;
+
+		NdisRawWritePortUlong(PCI_CONF_ADDRESS , PciCfgAddrPort+(Num4Bytes << 2));
+
+		NdisRawWritePortUchar(PCI_CONF_DATA, PciBridgeLinkCtrlReg);	
+		RT_TRACE(COMP_POWER, "PlatformDisableASPM():PciBridge BusNumber[%x], DevNumbe[%x], FuncNumber[%x], Write reg[%x] = %x\n",
+			priv->NdisAdapter.PciBridgeBusNum, priv->NdisAdapter.PciBridgeDevNum, priv->NdisAdapter.PciBridgeFuncNum, 
+			(priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10), PciBridgeLinkCtrlReg);
+		
+		udelay(50);
+#else
 		pci_write_config_byte(priv->bridge_pdev,priv->PciBridgeASPMRegOffset,PciBridgeLinkCtrlReg);
 		RT_TRACE(COMP_POWER, "Write reg[%x]=%x\n", (priv->PciBridgeASPMRegOffset), PciBridgeLinkCtrlReg);
 		udelay(100);
+#endif
 	}
 
-	PlatformSwitchDevicePciASPM(dev, priv->LinkCtrlReg);
+#ifdef RTL8192SE
+	PlatformSwitchDevicePciASPM(dev, LinkCtrlReg);
 
 	PlatformSwitchClkReq(dev, 0x0);
 	if (pPSC->RegRfPsLevel & RT_RF_OFF_LEVL_CLK_REQ)
 		RT_CLEAR_PS_LEVEL(pPSC, RT_RF_OFF_LEVL_CLK_REQ);
 	udelay(100);
+#endif
+
 }
 
 void PlatformEnableASPM(struct net_device *dev)
 {
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
-	PRT_POWER_SAVE_CONTROL pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->ieee80211->PowerSaveControl));
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
+	PRT_POWER_SAVE_CONTROL pPSC = (PRT_POWER_SAVE_CONTROL)(&(priv->rtllib->PowerSaveControl));
 	u16	ASPMLevel = 0;
+	u32	PciCfgAddrPort=0;
+	u8	Num4Bytes;
+	u8	uPciBridgeASPMSetting = 0;
+	u8	uDeviceASPMSetting = 0;
 
-	if (!priv->aspm_clkreq_enable) {
-		RT_TRACE(COMP_INIT, "%s: Fail to enable ASPM. Cannot find the Bus of PCI(Bridge).\n",\
-			       	__FUNCTION__);
+
+	if( (priv->NdisAdapter.BusNumber == 0xff && priv->NdisAdapter.DevNumber == 0xff && priv->NdisAdapter.FuncNumber == 0xff) ||
+		(priv->NdisAdapter.PciBridgeBusNum == 0xff && priv->NdisAdapter.PciBridgeDevNum == 0xff && priv->NdisAdapter.PciBridgeFuncNum == 0xff) )
+	{
+		RT_TRACE(COMP_INIT, "PlatformEnableASPM(): Fail to enable ASPM. Cannot find the Bus of PCI(Bridge).\n");
 		return;
 	}
 
+#ifdef RTL8192SE
+	if(priv->NdisAdapter.PciBridgeVendor != PCI_BRIDGE_VENDOR_INTEL )
+	{
+		RT_TRACE(COMP_POWER, "%s(): Dont modify ASPM for non intel chipset. For Bridge Vendor %d.\n"
+			,__func__,priv->NdisAdapter.PciBridgeVendor);
+		return;
+	}
+#endif
+
+#ifdef RTL8192CE
+	PciCfgAddrPort= (priv->NdisAdapter.PciBridgeBusNum << 16)|(priv->NdisAdapter.PciBridgeDevNum<< 11)
+							|(priv->NdisAdapter.PciBridgeFuncNum <<  8)|(1 << 31);
+	Num4Bytes = (priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10)/4;
+	NdisRawWritePortUlong(PCI_CONF_ADDRESS , PciCfgAddrPort+(Num4Bytes << 2));
+
+	uPciBridgeASPMSetting = priv->NdisAdapter.PciBridgeLinkCtrlReg |priv->RegHostPciASPMSetting;
+	
+#ifdef RTL8192SE
+	if(priv->CustomerID == RT_CID_TOSHIBA && priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_AMD 
+		&& pPSC->RegAMDPciASPM)
+	{
+		if(priv->NdisAdapter.PciBridgeLinkCtrlReg & BIT1)
+			uPciBridgeASPMSetting |= BIT0;
+	}
+	else
+#endif
+	if(priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_INTEL )
+		uPciBridgeASPMSetting &=  ~ BIT0;
+
+	NdisRawWritePortUchar(PCI_CONF_DATA, uPciBridgeASPMSetting);
+	RT_TRACE(COMP_INIT, "PlatformEnableASPM():PciBridge BusNumber[%x], DevNumbe[%x], FuncNumber[%x], Write reg[%x] = %x\n",
+		priv->NdisAdapter.PciBridgeBusNum, priv->NdisAdapter.PciBridgeDevNum, priv->NdisAdapter.PciBridgeFuncNum, (priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10), 
+		uPciBridgeASPMSetting);
+
+	udelay(50);
+#endif
+
 	ASPMLevel |= priv->RegDevicePciASPMSetting;
-	PlatformSwitchDevicePciASPM(dev, (priv->LinkCtrlReg | ASPMLevel));
+	uDeviceASPMSetting = priv->NdisAdapter.LinkCtrlReg;
+
+
+#ifdef RTL8192SE
+	if(priv->CustomerID == RT_CID_TOSHIBA && priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_AMD)
+	{
+		if(priv->NdisAdapter.LinkCtrlReg & BIT1)
+			uDeviceASPMSetting |= BIT0;
+		else 
+			uDeviceASPMSetting |= ASPMLevel;			
+	}
+	else
+#endif
+		uDeviceASPMSetting |= ASPMLevel;
+	
+	PlatformSwitchDevicePciASPM(dev, uDeviceASPMSetting);
 
 	if (pPSC->RegRfPsLevel & RT_RF_OFF_LEVL_CLK_REQ) {
 		PlatformSwitchClkReq(dev,(pPSC->RegRfPsLevel & RT_RF_OFF_LEVL_CLK_REQ) ? 1 : 0);
@@ -605,17 +807,42 @@ void PlatformEnableASPM(struct net_device *dev)
 	}
 	udelay(100);
 
+#ifdef RTL8192SE
+#if 0
 	pci_write_config_byte(priv->bridge_pdev,priv->PciBridgeASPMRegOffset,\
-			((priv->PciBridgeLinkCtrlReg | priv->RegHostPciASPMSetting)&~BIT0)&0xff);
+			((priv->NdisAdapter.PciBridgeLinkCtrlReg | priv->RegHostPciASPMSetting)&~BIT0)&0xff);
 	RT_TRACE(COMP_INIT, "Write reg[%x] = %x\n",
 		priv->PciBridgeASPMRegOffset, 
-		(priv->PciBridgeLinkCtrlReg|priv->RegHostPciASPMSetting));
+		(priv->NdisAdapter.PciBridgeLinkCtrlReg|priv->RegHostPciASPMSetting));
+#else
+	PciCfgAddrPort= (priv->NdisAdapter.PciBridgeBusNum << 16)|(priv->NdisAdapter.PciBridgeDevNum<< 11)|(priv->NdisAdapter.PciBridgeFuncNum <<  8)|(1 << 31);
+	Num4Bytes = (priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10)/4;
+	NdisRawWritePortUlong(PCI_CONF_ADDRESS , PciCfgAddrPort+(Num4Bytes << 2));
+
+	uPciBridgeASPMSetting = priv->NdisAdapter.PciBridgeLinkCtrlReg |priv->RegHostPciASPMSetting;
+	
+	if(priv->CustomerID == RT_CID_TOSHIBA && priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_AMD 
+		&& pPSC->RegAMDPciASPM)
+	{
+		if(priv->NdisAdapter.PciBridgeLinkCtrlReg & BIT1)
+			uPciBridgeASPMSetting |= BIT0;
+	}
+	else if(priv->NdisAdapter.PciBridgeVendor == PCI_BRIDGE_VENDOR_INTEL )
+		uPciBridgeASPMSetting &=  ~ BIT0;
+
+	NdisRawWritePortUchar(PCI_CONF_DATA, uPciBridgeASPMSetting);
+	RT_TRACE(COMP_INIT, "PlatformEnableASPM():PciBridge BusNumber[%x], DevNumbe[%x], FuncNumber[%x], Write reg[%x] = %x\n",
+		priv->NdisAdapter.PciBridgeBusNum, priv->NdisAdapter.PciBridgeDevNum, priv->NdisAdapter.PciBridgeFuncNum, (priv->NdisAdapter.PciBridgePCIeHdrOffset+0x10), 
+		uPciBridgeASPMSetting);
+#endif
+#endif
+
 	udelay(100);
 }
 
 u32 PlatformResetPciSpace(struct net_device *dev,u8 Value)
 {
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
 
 	pci_write_config_byte(priv->pdev,0x04,Value);	
 
@@ -625,7 +852,7 @@ u32 PlatformResetPciSpace(struct net_device *dev,u8 Value)
 bool PlatformSetPMCSR(struct net_device *dev,u8 value,bool bTempSetting)
 {
 	bool bResult = false;
-	struct r8192_priv *priv = (struct r8192_priv *)ieee80211_priv(dev);
+	struct r8192_priv *priv = (struct r8192_priv *)rtllib_priv(dev);
 	u8  Buffer;
 	bool bActuallySet=false, bSetFunc=false;
 	unsigned long flag;
